import XPILib
import sys
import os
import glob

def check_file_number():
    if len(list_mzmls) < min_mzml:
        print("To draw Standard Curve, you need at least {0} mzML files, but you have {1} file(s)".format(
            min_mzml, len(list_mzmls)))
        # exit()
    else:
        pass


#########################
# Start Main program    #
#########################
if __name__ == "__main__":
    # These were used for XPI paper
    # check_hdl_ylim_ion_filtering and check_hdl_ylim
    XPILib.is_hdl = False
    XPILib.regression_flag = False

    # Check system argument and get data directory
    if len(sys.argv) < 3:
        XPILib.UserMessageXPIViz("Requires two command arguments")
    data_directory = sys.argv[1]
    mode = sys.argv[2]

    try:
        figure_size = sys.argv[3]
        if figure_size.lower() == 'small':
            # You can adjust the figure size by setting flag_figure_size = True
            XPILib.flag_figure_size = True
            XPILib.fixed_height = 2
            XPILib.fixed_width = 3
        else:
            XPILib.flag_figure_size = False

    except IndexError:
        flag_figure_size = False

    # Get the path of XPIViz_config file
    file_config_viz = os.path.join(data_directory, XPILib.file_viz_config)

    # print the program information
    print(XPILib.pgm_info)

    # Get mzML files
    list_mzmls = glob.glob("%s/*.mzML" % (data_directory))
    min_mzml = 3

    # Parse xpiviz_config file
    XPILib.check_file(file_config_viz)
    config_XPIViz = XPILib.XPIVizConfig(file_config_viz)

    # Check intensity-picking method
    picking_method = config_XPIViz.intensity_picking_method
    print("Selected quantification method: {0}".format(picking_method))
    if XPILib.check_intensity_picking_method(picking_method):
        XPILib.UserMessageXPIViz("Wrong [intensity-picking-method]: {0}".format(picking_method))

    # Parse XPI_output_time.txt to get path of XPIQuant_config.txt
    file_config_quant = XPILib.find_data_directory_from_time_file(data_directory)

    # Parse xpiquant_config file
    config_quant_obj = XPILib.XPIQuantConfig(file_config_quant)
    max_length_fragment = int(config_quant_obj.data[XPILib.colname_max_length_fragment])
    ms2_window = float(config_quant_obj.data[XPILib.colname_ms2_window])
    list_labeling = XPILib.sort_dict_keys(config_quant_obj.labeling.data)

    ttr = config_quant_obj.data[XPILib.colname_ttr]
    list_labels = list(config_quant_obj.labeling.data.keys())
    list_labels.sort()

    # Parse XPI_transitions.txt
    transition_file = os.path.join(data_directory, XPILib.file_transition)
    labeling_obj = config_quant_obj.labeling
    transition_obj = XPILib.Transition(transition_file, labeling_obj)

    # Parse XPI_output_all.txt
    file_xpi_output_all = os.path.join(data_directory, XPILib.file_all)
    xpi_all_obj = XPILib.XPIOutAll(file_xpi_output_all, config_XPIViz)

    # Parse XPI_output_peaks.txt
    file_xpi_peak = os.path.join(data_directory, XPILib.file_peak_info)
    xpi_peak_obj = XPILib.XPIPeak(file_xpi_peak)

    # Wrtie XPI_output_4check.txt based on the current peak picking information
    XPILib.update_xpi_4check(data_directory, list_labeling, xpi_all_obj, xpi_peak_obj, config_quant_obj)

    # Get the ratio User selected
    file_4check_path = os.path.join(data_directory, XPILib.file_check)
    xpi_4check = XPILib.XPI4Check(file_4check_path, XPILib.get_ratio_name(config_XPIViz.data),
                                  config_XPIViz.standard_label, ttr=ttr)

    if (mode.lower() == XPILib.mode_for_filter):
        # Get sum of ratios and sum of M0 intensity from XPICheck
        dict_tr2sum_ri = XPILib.check_sum_ratio_of_fragment(data_directory, xpi_4check.data[picking_method], config_XPIViz)
    else:
        dict_tr2sum_ri = XPILib.get_tr_id_selection_info(data_directory)

    # Check if there's at least one valid PRM ion
    cnt_valid_tr_id = 0
    for tran_id in dict_tr2sum_ri:
        if not dict_tr2sum_ri[tran_id]['sum_ratio']:
            cnt_valid_tr_id += not dict_tr2sum_ri[tran_id]['sum_intensity']
    if cnt_valid_tr_id == 0:
        XPILib.UserMessageXPIViz.print_message("There's no valid PRM ions. Adjust your 'Filtering' threshold.")
    # Execute appropriate function for the user-input mode
    if mode.lower() == XPILib.mode_for_3d_profile:
        dir_3d_mass_profile = XPILib.get_path_for_plot(data_directory, XPILib.dir_3d_profile)
        XPILib.draw_3d_mass_profile(xpi_all_obj, config_XPIViz, xpi_peak_obj, dir_3d_mass_profile, ms2_window, list_labels)
        
    elif mode.lower() == XPILib.mode_for_linegraph_ions:
        dir_dir_ion_lines = XPILib.get_path_for_plot(data_directory, XPILib.dir_ion_lines)
        XPILib.draw_line_mass_profiles(xpi_all_obj, config_XPIViz, xpi_peak_obj, dir_dir_ion_lines, ms2_window, list_labels)
        
    # Protein scatter plots
    elif mode.lower() == XPILib.mode_for_scatter_protein:
        check_file_number()
        dir_scatter_prot = XPILib.get_path_for_plot(data_directory, XPILib.dir_scatter_protein)
        # Draw protein scatter plot
        XPILib.draw_scatter_filtering_plot_for_proteins("time_series", dir_scatter_prot, xpi_4check, picking_method,
                                                  config_XPIViz, dict_tr2sum_ri, transition_obj)
        # Draw protein filling plot
        XPILib.draw_scatter_filtering_plot_for_proteins("filling", dir_scatter_prot, xpi_4check, picking_method,
                                                  config_XPIViz, dict_tr2sum_ri, transition_obj)
        # Draw protein error_bar plot
        XPILib.draw_scatter_filtering_plot_for_proteins("error_bar", dir_scatter_prot, xpi_4check, picking_method,
                                                  config_XPIViz, dict_tr2sum_ri, transition_obj)

    # Box-plots and ratio-intensity plot
    elif mode.lower() == XPILib.mode_for_filter:
        pg_num_box = 1
        pg_box = XPILib.Progress(2, "Drawing validation plots...")
        dir_scatter_filter = XPILib.get_path_for_plot(data_directory, XPILib.dir_scatter_filter)

        # Draw intensity vs ratio plot
        # flag_valid = False  # If you want to see all the transitions, set it False
        flag_valid = True # If you want to see the filtered transitions only, set it True
        XPILib.draw_scatter_filtering_plot_for_proteins('enrichment_filter', dir_scatter_filter, xpi_4check, picking_method,
                                                  config_XPIViz, dict_tr2sum_ri, transition_obj, flag_show=flag_valid)
        XPILib.draw_scatter_filtering_plot_for_proteins('enrichment_filter_log', dir_scatter_filter, xpi_4check,
                                                  picking_method, config_XPIViz, dict_tr2sum_ri, transition_obj,
                                                  flag_show=flag_valid)
        
        # Calculate R2 for all the intensity-picking-methods and draw box-graphs
        pg_box.print_progress(pg_num_box)
        XPILib.draw_box_plot(data_directory, dir_scatter_filter, config_XPIViz, ttr, dict_tr2sum_ri, metrics='pcc')
        pg_num_box += 1

    # Peptide scatter plots
    elif mode.lower() == XPILib.mode_for_scatter_peptide:
        check_file_number()
        dir_scatter_pept = XPILib.get_path_for_plot(data_directory, XPILib.dir_scatter_peptide)
        # Draw peptide scatter plot
        XPILib.draw_time_series_plot_for_peptides("scatter", dir_scatter_pept, xpi_4check, picking_method,
                                                  config_XPIViz, dict_tr2sum_ri, max_length_fragment, transition_obj)
        # Draw peptide filling plot
        XPILib.draw_time_series_plot_for_peptides("filling", dir_scatter_pept, xpi_4check, picking_method,
                                                  config_XPIViz, dict_tr2sum_ri, max_length_fragment, transition_obj)

    # Standard curves
    elif mode.lower() == XPILib.mode_for_standard_curve:
        check_file_number()
        dir_std_curves = XPILib.get_path_for_plot(data_directory, XPILib.dir_std_curve)
        XPILib.draw_standard_curve(dir_std_curves, xpi_4check, picking_method, config_XPIViz, dict_tr2sum_ri)

    # 2-dimensional mass profile
    elif mode.lower() == XPILib.mode_for_2d_profile:
        dir_2d_mass_profile = XPILib.get_path_for_plot(data_directory, XPILib.dir_2d_profile)
        XPILib.draw_2d_mass_profile(xpi_all_obj, config_XPIViz, list_labels, xpi_peak_obj, ms2_window,
                                    dir_2d_mass_profile)

    # XPI Output report
    elif mode.lower() == XPILib.mode_for_output:
        XPILib.print_xpi_output(transition_obj, data_directory, list_labeling, picking_method, config_quant_obj,
                                xpi_all_obj)
        XPILib.print_xpi_output_selected(data_directory)

    # Error message
    else:
        XPILib.UserMessageXPIViz("Wrong option. {0}".format(mode))

    print("All the results are generated at {0}".format(data_directory))
